use eframe::egui::{Color32, Context, Label, Response, Ui, Widget};
use eframe::{egui, Frame};
use reqwest::Url;
use std::ops::Sub;
use std::str::FromStr;
use std::time::{Duration, Instant};

pub struct MonitorTarget {
    url: String,
    last_update: Instant,
    last_status: bool,
}

impl MonitorTarget {
    pub fn new<S: Into<String>>(url: S) -> MonitorTarget {
        MonitorTarget {
            url: url.into(),
            last_status: false,
            last_update: Instant::now().sub(Duration::new(30, 0)),
        }
    }

    pub fn check(&mut self) -> bool {
        if self.last_update.elapsed().as_secs() > 30 {
            self.last_status = reqwest::blocking::get(self.url.as_str()).is_ok();
            self.last_update = Instant::now();
        }
        self.last_status
    }
}

#[derive(Default)]
pub struct UptimeWidget {
    pub targets: Vec<MonitorTarget>,
    _url_input_cache: String,
}

impl eframe::App for UptimeWidget {
    fn update(&mut self, ctx: &Context, frame: &mut Frame) {
        egui::Window::new("Uptime Widget").show(ctx, |ui| {
            self.targets.retain_mut(|target| {
                ui.horizontal(|ui| {
                    ui.label(target.url.clone());
                    ui.label({
                        if target.check() {
                            format!("OK ({}s ago)", target.last_update.elapsed().as_secs())
                        } else {
                            format!("FAIL ({}s ago)", target.last_update.elapsed().as_secs())
                        }
                    });
                    !ui.button("x").clicked()
                })
                .inner
            });
            ui.horizontal(|ui| {
                ui.text_edit_singleline(&mut self._url_input_cache)
                    .on_hover_text("URL to add");
                if ui.button("Add URL").clicked() {
                    self.targets
                        .push(MonitorTarget::new(self._url_input_cache.clone()));
                    self._url_input_cache = String::default();
                }
            });

            if ui.is_visible() {
                ctx.request_repaint();
            }
        });
    }
}
