use std::collections::HashMap;

use crate::apps::clock::ClockWidget;
use crate::apps::greeting::Greeting;
use crate::apps::uptime::UptimeWidget;
use eframe::egui::accesskit::Role::Window;
use eframe::egui::panel::Side;
use eframe::egui::{CentralPanel, Id, SidePanel};
use eframe::{egui, App};

#[derive(Default)]
pub struct Dashboard {
    pub user_name: String,
    pub apps: HashMap<String, Box<dyn App>>,
}

impl Dashboard {
    pub fn has_app<S: Into<String>>(&self, name: S) -> bool {
        self.apps.contains_key(&name.into())
    }

    pub fn add_app<S: Into<String>>(&mut self, name: S, app: Box<dyn App>) {
        self.apps.insert(name.into(), app);
    }

    pub fn remove_app<S: Into<String>>(&mut self, name: S) {
        self.apps.remove(&name.into());
    }
}

macro_rules! register_app {
    ($dash:ident, $ui:ident, $name:literal, $app:expr) => {
        let button = $ui.button($name);
        if button.clicked() {
            if $dash.has_app($name) {
                $dash.remove_app($name);
            } else {
                $dash.add_app($name, Box::new($app));
            }
        }
        if $dash.has_app($name) {
            button.highlight();
        }
    };
}

impl eframe::App for Dashboard {
    fn update(&mut self, ctx: &egui::Context, frame: &mut eframe::Frame) {
        SidePanel::new(Side::Left, Id::new("side_panel")).show(ctx, |ui| {
            ui.heading("Apps");
            register_app!(self, ui, "Clock", ClockWidget);
            register_app!(self, ui, "Uptime Widget", UptimeWidget::default());
        });
        CentralPanel::default().show(ctx, |ui| {});
        self.apps.iter_mut().for_each(|(_, app)| {
            app.update(ctx, frame);
        })
    }
}
